'#Reference {A518A78A-ECF7-4AD8-AFF1-A7482FEA9A0F}#1.c#0#C:\FtgCode\Vb6Code\Index1.exe#FtgIndex1
' CodeV.BAS for FilmStar DESIGN
' Copyright 2005-2018 FTG Software Associates
'
' ********************************************************
'      Warning: Save modified program with new name!!!
' ********************************************************
'
' Creates coating files for Code V optical design program
' Please notify FTG Software if there are issues with your
' version of Code V.
'
' ********************************************************
'                          Notes
'
'  1. Requires DESIGN 2.40.0100 or newer
'  2. Make sure directory ..\Winfilm\CodeV exists
'  3. Max 21 wavelengths in dispersive files; wavelengths
'     in all index tables must be identical. To set this up,
'     use Functions...Convert Table in FilmStar INDEX.
'  4. DESIGN should be set in R/T/OD vs Wavelength
'     (not Wavelength Range or Wavelength List)
'  5. This macro changes designs to physical thickness
'     and removes any parentheses. So be careful about
'     hitting 'Save'.
'
Option Explicit
Option Base 1
Public pINDEX As FtgIndex1.clsBasic
DefInt i-N
DefSng A-H, O-Z
Public Const title$ = "Code V Coating File"

Public symb$(), ntyp$(), file$(), isDisp() As Boolean

Public Const sp$ = " "
Public Const pp$ = "'"
Public Const qt$ = Chr$(34)

Sub Main
    Dim wMin, wMax, wDel, dum$, s$, path$, sFile$
    Set pINDEX = New FtgIndex1.clsBasic ' Reference to FilmStar INDEX
    FileSaveAll     ' added 07.26.18
    AxesCopy True   ' Get wavelength range from current settings
    s$ = Clipboard$()
    Parse s$, dum$, wMin, vbTab
    Parse s$, dum$, wMax, vbTab
    Parse s$, dum$, wDel, vbTab
    If wDel < 0 Then
        MsgBox "Uncheck 'Inverse wavelength spacing'" & vbLf & "in Graph Axes and try again.", vbCritical, title$
        End
    End If

    ' Open .seq file with same name as FILM or FILM Archive
    ' Make sure that Winfilm\CodeV subdirectory exists

	If Not DirExists(PgmPath & "\CodeV") Then MkDir PgmPath & "\CodeV"
    Open PgmPath & "\CodeV\" & FilmName$ & ".seq" For Output As #1

    Print #1, "mul"
    Print #1, "mda"
    Print #1, "tit " & qt$ & FilmName$ & qt$
    Print #1, "wl " & tStr$(wMin) & sp$ & tStr$(wMax) ' Analysis wavelength range
    Print #1, "ang " & tStr$(Angle)
    AddMaterials
    AddLayers
	
    Print #1, vbCrLf & "mpl" ' Multilayer plot

    Select Case AxesYType    ' Spectrum type based on current program setting
    Case "R"
        Print #1, "rfl"
    Case "T", "O"
        Print #1, "trn"
    Case Else
        MsgBox "FilmStar DESIGN not set to Refl, Trns or OD!", vbCritical, title$
        End
    End Select

    Select Case Pol ' Polarization type based on current program setting
    Case "P"
        Print #1, "ppl"     ' P pol
    Case "S"
        Print #1, "spl"     ' S pol
    Case "R"
        Print #1, "ave"     ' Average
       'Print #1, "all"     ' Both polarizations
    End Select
    Print #1, "spa " & tStr$(wMin) & sp$ & tStr$(wMax) & sp$ & tStr$(wDel) ' Plot wavelength range
    Print #1, "go"  ' Execute the plot
    Print #1, "sav " & LglStr$(FilmName$)   ' Save design in Code V
    Print #1, "mex"
    Close #1        ' Done!!
    FileOpenAll
    MsgBox "Coating file " & PgmPath & "\CodeV\" & FilmName$ & ".seq saved successfully", vbInformation, "Code V"
End Sub

Sub AddLayers
    Dim i, k, s$
    Dim nLay, q(), qt$(), qd$()

    DesignConvert 1, "N"   ' Convert design to physical thickness in nm
    DesignReverse          ' Reverse design for Code V

    GetLayers nLay, q(), qt$(), qd$(), 1

    If nLay = 0 Then
        MsgBox "No thin film design defined!", vbCritical, title$
        End
    End If

    Print #1, vbCrLf & "pht yes"  ' Design is in nanometers
    For i = 1 To nLay
        s$ = "coa " & tStr$(q(i)) & " 100 "
        k = InStr(qd$(i), "|")
        If k > 0 Then   ' Numeric index values
            s$ = s$ & Left$(qd$(i), k - 1)
            If Val(Mid$(qd$(i), k + 1)) > 0 Then s$ = s$ & " " & Mid$(qd$(i), k + 1)
        Else
            s$ = s$ & pp$ & LglDsp$(qd$(i)) & pp$
        End If
        Print #1, s$
    Next i
    s$ = "inc "
    If isDisp(1) Then  ' incident medium (non absorbing)
        s$ = s$ & pp$ & LglDsp(file$(1)) & pp$
    Else
        s$ = s$ & tStr$(IndexCoeff(1, 1))
    End If
    Print #1, s$
    s$ = "sub "
    If isDisp(2) Then  ' substrate (non absorbing)
        s$ = s$ & pp$ & LglDsp(file$(2)) & pp$
    Else
        s$ = s$ & tStr$(IndexCoeff(2, 1))
    End If
    Print #1, s$
    DesignReverse ' restore original order
End Sub

Sub AddMaterials
    Dim i, j, n, dum$(), mic As Boolean, mwl!(), kDisp As Boolean
    Dim sw$, sn$, sk$

    Dim nk!()   ' index table array

    GetIndexList symb$(), ntyp$(), file$(), dum$()
    n = UBound(symb$)
    ReDim nval$(n, 2), isDisp(n)

    For i = 1 To UBound(symb$)
        If Trim$(file$(i)) = "" Then    ' numeric
            isDisp(i) = False
        Else
            isDisp(i) = True
            kDisp = True   ' Houston, we have dispersion!
            If Not mic Then       ' start Multilayer Index Catalog
                Print #1, vbCrLf & "mic"
                With pINDEX
                    .IndexOpen file$(i)
                    If .nIndex > 21 Then
                        MsgBox "Index Table " & file$(i) & " has more than 21 wavelengths!", vbCritical, title$
                        End
                    Else
                        ReDim mwl!(.nIndex)
                        nk = .nkTable
                        If Not mic Then ReDim mwl!(.nIndex)
                        For j = 1 To .nIndex
                            mwl!(j) = nk(j, 1)  ' get wavelengths
                        Next j
                        sw$ = "mwl"
                        For j = 1 To .nIndex
                            sw$ = sw$ & sp$ & tStr$(nk(j, 1))
                        Next j
                        Print #1, sw$
                        mic = True
                    End If
                End With
            End If
            With pINDEX
                .IndexOpen file$(i)
                If .nIndex <> UBound(mwl!) Then
                    MsgBox "Index Table " & file$(i) & " has invalid number of wavelengths!", vbCritical, title$
                    End
                End If
                nk = .nkTable
                sn$ = pp$ & LglDsp$(file$(i)) & pp$
                For j = 1 To .nIndex
                    If nk(j, 1) <> mwl!(j) Then
                        MsgBox "Index Table " & file$(i) & " has invalid wavelengths!", vbCritical, title$
                        End
                    End If
                    sn$ = sn$ & sp$ & tStr$(nk(j, 2))
                Next j
                Print #1, sn$

                For j = 1 To .nIndex
                    If nk(j, 3) > 0 Then
                        sk$ = "ext " & pp$ & LglDsp$(file$(i)) & pp$
                        Exit For
                    End If
                Next j
                If sk$ <> "" Then
                    For j = 1 To .nIndex
                        sk$ = sk$ & sp$ & tStr$(nk(j, 3))
                    Next j
                    Print #1, sk$
                End If
            End With
        End If
    Next i
    If kDisp Then Print #1, "end"
End Sub

Sub Parse (strin$, strout$, xout As Variant, delim$)
    Dim i
    strin$ = LTrim$((strin$))
    i = InStr(strin$, delim$)
    If i > 0 Then
        strout$ = LTrim$(Mid$(strin$, 1, i - 1))
        strin$ = Mid$(strin$, i + Len(delim$))
    Else
        strout$ = strin$
        strin$ = ""
    End If
    xout = Val(strout$)
End Sub

Function tStr$(ByVal v)
    tStr$ = Trim$(Str$(v))
End Function

Property Get FilmName$    ' returns FILM or FILM Archive file name
    If ArchiveMode Then FilmName = FileName(8) Else FilmName = FileName(7)
End Property

Function LglStr$(ByVal s$)   ' Legal Code V characters
    Dim i
    For i =1 To Len(s$)
        Select Case Mid$(s$, i, 1)
        Case "0" To "9", "_"
            If i = 1 Then
                MsgBox "Illegal string '" & s$ & "' (first character not alphabetic)!", vbCritical, title$
                End
            End If
        Case "A" To "Z", "a" To "z"
        Case Else
            Mid$(s$, i, 1) = "_"
        End Select
    Next i
    LglStr$ = s$
End Function

Function LglDsp$(ByVal s$) ' Limit of 6 char in Code V dispersion names
    s$ = LglStr$(s$)
    If Len(s$) < 7 Then
        LglDsp$ = s$
    Else
        LglDsp$ = Left$(s$, 6)
    End If
End Function

Function DirExists(ByVal DName$) As Boolean
    Dim sdum$
    On Error Resume Next
    If DName$ <> "" Then
        If Right(DName$, 1) <> "\" Then DName = DName & "\"
        sdum = Dir$(DName & "*.*", vbDirectory)
        DirExists = sdum <> ""
    Else
        DirExists = False
    End If
End Function
