' Gaussian.bas for FilmStar BASIC
' Copyright 1995-2000 FTG Software Associates
'
' ****************************************************
'    Warning: Save modified program with new name!!!
' ****************************************************
'
' Try this with FILM file FNUM.
'
' This program simulates the performance of a filter with a non-ideal
' monochromator with Gaussian beam shape. It does this by averaging the
' calculation over a moving wavelength interval.
'
' The calculation interval must be < bandwidth. For a bandwidth of 1 nm
' of 1 nm use a calculation interval ~ 0.1 - 0.2 nm.
'
' To maximize calculation speed select graph axes to calculate only in
' the region of peaks of interest.
'
' To run this program as a macro, insert the following under Evaluate...Macro...Edit
' in DESIGN's Main Menu (adjusting if your winfilm directory is different)
'
'   Description   Gaussian Mono Beam
'   Command       basrun c:\winfilm\basic32\gaussian.bas;
'
DefInt I-N
DefSng A-H, O-Z
Dim bwidth$    ' bandwidths separated by commas, ie. 1, 3, 5
Dim kZero      ' plot initial (zero bandwidth) spectrum
'
Sub Main
    bwidth$ = "1, 3, 5"   ' initialize
    kZero = 1             ' initialize
    Do
        GetWidth
        Evaluate    
        PlotActivate    ' activate FSPlot Module
    Loop
End Sub

Sub GetWidth             ' dialog box for entering bandwidth
    On Error GoTo Quit   ' cancel button causes error
' Highlight the lines Begin Dialog...End Dialog and activate the
' built-in dialog editor. The dialog box appears in the editor.
    Begin Dialog UserDialog 440,126,"Gaussian Light Source"
        Text 25,23,105,15,"&Bandwidth (nm)"
        TextBox 140,21,135,19,.WidthText
        OKButton 315,20,90,20
        CancelButton 315,55,90,20
        Text 50,49,220,28,"Enter up to six monochromator bandwidths separated by commas"
        CheckBox 30,91,280,21,"Show initial (&zero bandwidth) spectrum",.ZeroCheck
    End Dialog
    Dim dlg As UserDialog
    dlg.WidthText = bwidth$
    dlg.ZeroCheck = kZero
    Dialog dlg
    bwidth$ = dlg.WidthText
    kZero = dlg.ZeroCheck
    Exit Sub
Quit:
    End     'stop program
End Sub

Sub Evaluate   ' does the calculation
    Dim kErr As Boolean
    Dim xdata(), ydata(), ytemp()
    Busy True   ' hourglass
    Calculate   ' do calculation
    PlotClose   ' make sure FSPlot window closed
    PlotAnnotate False  ' no annotations (before PlotActivate!)
    PlotLegendOn True   ' legends on (before PlotActivate!)
    PlotActivate    ' open FSPlot window
    If kZero Then
        PlotLegend(1) = "0 nm"
        PlotNext
        kPlot = 1   ' first plot
    End If
    btemp$ = bwidth$
    Do While btemp$ <> ""
        Parse btemp$, dum$, bw, ","  ' get next bandwidth
        Busy True
        MonoLight bw, kErr  ' do calculation
        If kErr Then
            Beep
            Exit Do
        End If
        kPlot = kPlot + 1
        PlotLegend(kPlot) = CStr(bw) & " nm"
        PlotNext
        If kPlot = 12 Then Exit Do    ' 12 legends maximum
    Loop
    Busy False
End Sub

Sub Parse (strin$, strout$, xout As Variant, delim$)
    strin$ = LTrim$((strin$))
    i = InStr(strin$, delim$)
    If i > 0 Then
        strout$ = LTrim$(Mid$(strin$, 1, i - 1))
        strin$ = Mid$(strin$, i + Len(delim$))
    Else
        strout$ = strin$
        strin$ = ""
    End If
    xout = Val(strout$)
End Sub
