' TFCALC.BAS for FilmStar DESIGN 32-bit
' Copyright 2001-2010 FTG Software Associates
'
'    01.20.2001...Initial version
'    08.02.2010...Major revision for DESIGN 2.61.0312
'
' ****************************************************
'    Warning: Save modified program with new name!!!
' ****************************************************
'
'#Uses "GetWindowTitle.bas"
'
' Imports TFCalc designs which have been copied to the clipboard or
' while TFCalcf is open or saved as text files in the TFCalc Layers
' dialog
'
'    0. Before starting, make sure that all TFCalc material files
'       have been converted to FilmStar .itw files in FilmStar INDEX
'       (using INDEX BASIC macro TFCalcMatl.bas) and are include in
'       the Film Indices dialog Dispersion column.
'
'    1. Enter TFCalc material names in the Dispersion column in
'       Film Indices.
'
'    2. Set Type to Phys (physical thickness) in Film Indices and
'       Layer Physical Thickness = nm in Setup Parameters
'
'    3. If selecting 'Copy from TFCalc', make sure TFCalc is open
'       and the Layers dialog is displayed. This method provides the
'       opportunity to verify that design was correctly imported.
'
Option Explicit
Option Base 1
DefInt i-N
DefSng A-H, O-Z

Public TFCFile$
Const Title$ = "TFCalc Design Import"

Sub Main
    Dim d$, kErr As Boolean, kOpt, File$
    MainEvents = True
    On Error Resume Next
    kOpt = CInt(IniString("TFCalc", "DesignOption"))
    On Error GoTo Quit
    Begin Dialog UserDialog 335,129,"TFCalc -> FilmStar DESIGN" ' %GRID:5,3,1,1
        OptionGroup .grpDesign
            OptionButton 50,21,250,14,"&Copy from TFCalc",.optClip
            OptionButton 50,49,250,14,"&Open from disk",.optOpen
        OKButton 75,87,80,24
        CancelButton 185,87,80,24
    End Dialog
    Dim dlg As UserDialog
    dlg.grpDesign = kOpt
    Dialog dlg
    kOpt = dlg.grpDesign
    Select Case kOpt
    Case 0
        On Error GoTo TFCalcNA
        AppActivate "Layers"
        SendKeys "%OY", True
        If Not Convert(Clipboard$()) Then End
        AppActivate "TFCalc"    ' get file name from TFCalc main window
        FileName$(8) = Mid$(WindowTitle(True), 10)
    Case 1
        If DesDisk(d$) Then
            If Not Convert(d$) Then End
        End If
    End Select
    WriteIniString "TFCalc", "DesignOption", CStr(kOpt)
    AppActivate "FilmStar DESIGN"
    ArchiveMode = True
    AxesDraw
    CalcPlot
    SendKeys "{F12}"    ' FILM Archive save prompt
Quit:
    End
TFCalcNA:
    MsgBox "TFCalc Layers dialog not available", vbCritical, Title$
    End
End Sub

Private Function DesDisk(d$) As Boolean
    Dim i, s$, fNam$, Path$
    On Error GoTo DiskErr
    Path$ = IniString("TFCalc", "DesignPath")
    If Path$ = "" Then Path$ = "c:\TFCalc\COATINGS"

    TFCFile$ = GetFilePath$ (,"txt", Path$, "Open TFCalc Layer Data", 1)

    FilePath TFCFile$, Path$, fNam$
    Open TFCFile$ For Input As #1
    d$ = ""
    Do
        Line Input #1, s$
        d$ = d$ & s$ & vbCrLf
    Loop Until EOF(1)
    Close #1
    DesDisk = True
    FileName$(8) = fNam$
    WriteIniString "TFCalc", "DesignPath", Path$
    Exit Function
DiskErr:
    Close #1    ' just in case
    DesDisk = False
End Function

Private Function Convert(ByVal TFCDes$) As Boolean
    Dim d$, dFS$, i, j, nLines, s$, sdum$, slin$, xdum
    Dim nSymb, matl$, thk, nLay, ok As Boolean
    Dim symb$(), disp$(), fName$
    s$ = Review(1)     ' review design in format 1
    Do
        Parse s$, slin$, xdum, vbCrLf
        If InStr(slin$, "Symb   $Funct") > 0 Then Exit Do
    Loop
    SkipParse s$, 1, vbCrLf ' Skip 1 line
    For i = 1 To 2  ' AIR SUB
        Parse s$, slin$, xdum, vbCrLf
        ReDim Preserve symb$(i), disp$(i)
        Parse slin$, symb$(i), xdum, " "
        If Mid$(slin$, 2, 1) = " " Then
            MsgBox "Cannot convert - no dispersion file for " & symb$(i), vbCritical, Title$
            Convert = False
            Exit Function
        End If
        Parse slin$, disp$(i), xdum, " "
    Next i
    SkipParse s$, 1, vbCrLf ' Skip 1 line
    Do
        Parse s$, slin$, xdum, vbCrLf
        If slin$ = "" Then Exit Do
        ReDim Preserve symb$(i), disp$(i)
        Parse slin$, symb$(i), xdum, " "
        If Right$(symb$(i), 1) <> "*" Then
            MsgBox "Material type '" & symb$(i) & "' not physical thickness!", vbCritical
            Convert = False
            Exit Function
        End If
        If Mid$(slin$, 2, 1) = " " Then
            MsgBox "Cannot convert - no dispersion file for Index Symbol " & Left$(symb$(i), 1), vbCritical, Title$
            Convert = False
            Exit Function
        End If
        Parse slin$, disp$(i), xdum, " "
        i = i + 1
    Loop
    nSymb = i - 1  ' number of dispersion files in Film Indices dialog
    If Trim$(TFCDes$) = "" Then
        MsgBox "No design to convert", vbCritical, Title$
        Convert = False
        Exit Function
    End If
    nLay = Occurences(TFCDes$, vbCrLf)
    If nLay = 0 Then
        MsgBox "Cannot convert - invalid design", vbCritical, Title$
        Convert = False
        Exit Function
    End If
    For i=1 To nLay
        Parse TFCDes$, d$, xdum, vbCrLf
        Parse d$, matl$, xdum, vbTab
        Parse d$, sdum$, thk, vbTab  ' get thickness
        ok = False
        For j = 3 To nSymb   ' assigns index symbol to material name
            If UCase$(Trim$(matl$)) = UCase$(disp(j)) Then
                dFS$ = dFS$ & " " & Format$(thk) & Left(symb$(j), 1)
                ok = True
                Exit For
            End If
        Next j
        If Not ok Then
            MsgBox matl$ & " undefined in Film Indices dialog", vbCritical
            Convert = False
            Exit Function
        End If
    Next i
    Design = dFS$
    Convert = True
End Function

Private Function Occurences(ByVal t$, s$) As Integer
    Dim klen&, kpos&, ktot
    klen = Len(s$)  'number of occurences of s$ in t$
    Do
        kpos = InStr(t$, s$)
        If kpos = 0 Then
            Exit Do
        Else
            ktot = ktot + 1
            t$ = Mid$(t$, kpos + klen)
        End If
    Loop
    Occurences = ktot
End Function

Private Sub Parse (strin$, strout$, xout As Variant, ByVal delim$)
    Dim i
    strin$ = LTrim$((strin$))
    i = InStr(strin$, delim$)
    If i > 0 Then
        strout$ = LTrim$(Mid$(strin$, 1, i - 1))
        strin$ = Mid$(strin$, i + Len(delim$))
    Else
        strout$ = strin$
        strin$ = ""
    End If
    xout = Val(strout$)
End Sub

Private Sub SkipParse(s$, ByVal n, ByVal delim$)
    Dim i, dum$, xdum
    ' Skips n fields in s$
    For i = 1 To n
        Parse s$, dum$, xdum, delim$
    Next i
End Sub

Private Sub FilePath(ByVal f$, Path$, fNam$)
    Dim k, r$
    k = InStr(f$, "\")
    If k = 0 Then
        k = InStr(f$, ":")
        If k = 0 Then
            Path$ = ""
            fNam$ = f$
        Else
            Path$ = Mid$(f$, 1, 2)
            fNam$ = Mid$(f$, 3)
        End If
    Else
        k = InStrRev(f$, "\")   ' first occurence of \ from end of string
        Path$ = Mid$(f$, 1, k - 1)
        fNam$ = StripExt(Mid$(f$, k + 1))
    End If
End Sub

Private Function StripExt$(ByVal t$)    ' strip off file extension
    On Error Resume Next        ' cannot just search for "." in long file name
    If InStr(t$, ".") = 0 Then
        StripExt$ = t$      ' no extension
    Else
        StripExt$ = Left$(t$, InStrRev(t$, ".") - 1)
    End If
End Function
