'#Reference {A518A78A-ECF7-4AD8-AFF1-A7482FEA9A0F}#1.c#0#C:\FtgCode\Vb6Code\Index1.exe#FtgIndex1
' ZEMAX.BAS for FilmStar DESIGN 32-bit
' Copyright 2000-2018 FTG Software Associates
'
' 07.14.05...Fixed problem when coating file does not already exist
'            k is now < 0, "added by FilmStar" comments included
'            Note: Requires INDEX 2.40.0104 or newer
'
' 11.18.05...TABLE updated for latest Zemax version
'            Note: Uses Sub GetTableData added in DESIGN 2.40.0306
'
' 02.28.18...Remove TABLE capacity limitation of 100 wavelengths
'
' 07/26/18...FileSaveAll and FileOpenAll added
'
' ****************************************************
'    Warning: Save modified program with new name!!!
' ****************************************************
'
' Creates/updates coating files for Zemax optical design program
' Please notify FTG Software if there are issues with your version
' of Zemax
'
Option Explicit

' If there is an error in the next line, it is likely that
' the reference has been lost. Click Edit...References and
' re-establish the reference to FtgIndex1

Public pINDEX As FtgIndex1.clsBasic
DefInt i-N
DefSng A-H, O-Z

Sub Main
    Dim sOut$, file$, fname$
    Dim i, t!, nLay, q(), qt$(), qd$(), s$
    Dim Wave#, wMin#, wMax#, wDel#
    Dim xang#, AngList$, dum$, Taper$, WavRng$
    Dim Rs#, Rp#, Ts#, Tp#, Ars#, Arp#, Ats#, Atp#

    Set pINDEX = New FtgIndex1.clsBasic

    Const title$ = "Zemax Coating File"

    ' TABLE Format$
    Const f1$  = "0.000"    ' Wavelength
    Const f2$ = "0.0000"    ' Refl/Trns (0-1)
    Const f3$ = "0.00"      ' Phase angles

    ' File Path - change as required
    Const fPath = PgmPath & "\Zemax"    ' usually c:\winfilm\zemax

    On Error GoTo Quit
    file = GetFilePath$(,"dat", fPath$, "Open " & title$, 1)     ' cancel quits program
    On Error Resume Next
    If FileLen(file) > 0 Then FileCopy file, Left$(file, Len(file)-4) & ".bak"  ' just in case
    On Error GoTo Quit
    Do
        On Error GoTo Quit
        Begin Dialog UserDialog 370,126, title$ ' %GRID:5,3,1,1
            Text 25,9,315,12,file,.Text1
            GroupBox 30,30,200,81,"Add to file",.GroupBox1
            OptionGroup .Actions
                OptionButton 55,51,140,15,"Material",.optMATE
                OptionButton 55,69,120,15,"Design",.optCOAT
                OptionButton 55,87,155,15,"Table Data",.optTABLE ' version 10
            OKButton 260,39,80,21
            CancelButton 260,72,80,21
        End Dialog
        Dim dlg As UserDialog
        Dialog dlg
        Select Case dlg.actions
        Case 0
            GetMaterial sOut, fname
            Open PgmPath & "\temp.dat" For Output As #1
            Print #1, "! Material added by FilmStar " & Now
            Print #1, "MATE " & fname
            Print #1, sOut & vbCrLf
            If FileExists(file) Then
                Open file For Input As #2
                Do
                    Input #2, sOut
                    Print #1, sOut
                Loop Until EOF(2)
                Close
                Kill file
            Else
                Close #1
            End If
            FileCopy PgmPath & "\temp.dat", file
            MsgBox "Material '" & fname & "' added", vbInformation, title$
        Case 1
            If ArchiveMode Then
                fname = FileName(8) ' FILM Archive name
            Else
                fname = FileName(4) ' Design name {use FileName(7) for FILM name}
            End If
            fname = Replace(InputBox("Design (COAT) name?", title$, Replace(fname, " ", "_")), " ", "_")
            If fname <> "" Then
                FileSaveAll
                DesignConvert 1, "M"    ' convert to physical thickness in microns
                Taper = Replace(InputBox("Taper name for " & fname$ & "?" & vbCrLf & vbCrLf & "Enter blank {OK} if no taper; {Cancel} aborts", title$, Taper), " ", "_")
                GetLayers nLay, q(), qt(), qd(), 1
                If nLay > 0 Then
                    Open file For Append As #1
                    Print #1, "! Design added by FilmStar " & Now
                    Print #1, "COAT " & fname
                    If Taper = "" Then
                        For i = nLay To 1 Step -1
                            Print #1, qd(i) & cRnd(q(i), 6) & " 1 "
                        Next i
                    Else
                        For i = nLay To 1 Step -1
                            Print #1, qd(i) & cRnd(q(i), 6) & " 1 1 " & Taper
                        Next i
                    End If
                    Print #1, vbCrLf
                    Close #1
                    MsgBox "Design '" & fname & "' added", vbInformation, title$
                    FileOpenAll
                Else
                    MsgBox "Invalid design - no layers", vbExclamation, title$
                End If
            End If
        Case 2
            fname = Replace(InputBox("Table name?", title$, fname), " ", "_")
            If fname <> "" Then
                Open file For Append As #1
                Print #1, "! Table added by FilmStar " & Now
                Print #1, "TABLE " & fname
                If AngList$ = "" Then AngList$ = "0 10 20 30"
                AngList$ = InputBox("Angle list (space delimited)?", title$, AngList$)

                If WavRng$ = "" Then WavRng$ = "400 800 5"
                WavRng$ = InputBox("Wavelength range in nm (Min Max Delta)?", title$, WavRng$)

                ' NOTE: Zemax format is WAVE Rs Rp Ts Tp Ars Arp Ats Atp
                Parse WavRng$, dum, wMin, " "
                Parse WavRng$, dum, wMax, " "
                Parse WavRng$, dum, wDel, " "
                Busy True
                i = TypeSide2
                t = SubThick
                TypeSide2 = 1   ' required setting
                SubThick = 0    ' ignores any substrate bulk absorption
                Do
                    Parse AngList$, dum$, xang, " "
                    Print #1, "ANGL" & Str(xang)
                    For Wave = wMin To wMax Step wDel
                        GetTableData Wave, xang, Rs, Rp, Ts, Tp, Ars, Arp, Ats, Atp
                        s$ = "WAVE " & Format$(0.001* Wave, f1$) & " " & Format$(Rs, f2$) & " " & Format$(Rp, f2$) & " " & Format$(Ts, f2$) & " " & Format$(Tp, f2$) _
                              & " " & Format$(Ars, f3$) & " " & Format$(Arp, f3$) & " " & Format$(Ats, f3$) & " " & Format$(Atp, f3$)
                        Print #1, Replace(s$, ",",  ".")    ' in case European number format is used
                    Next Wave
                Loop Until AngList$ = ""
                TypeSide2 = i   ' restore original settings
                SubThick = t
                Busy False
                Close #1
                MsgBox "Table '" & fname & "' added", vbInformation, title$
            End If
        End Select
    Loop
Quit:
    Busy False
    Set pINDEX = Nothing
    End
End Sub

Sub GetMaterial(sInx As String, file As String)
    Static path As String
    If path = "" Then path = FilePath(9)
    file = GetFilePath$(,"itw", path$, "Open Index Table", 0)
    If file = "" Then
        Exit Sub
    Else
        With pINDEX
            .IndexOpen file
            sInx = .IndexList(,,True)   ' True gives k < 0
        End With
        file = GetName(file)
   End If
End Sub

Function GetName(ByVal f As String) As String
    ' Returns name used in Zemax header
    Dim k, fname As String, r As String
    k = InStr(f, "\")
    r = StrReverse(f)
    k = Len(f) - InStr(r, "\") + 1
    fname = Mid$(f$, k + 1)
    GetName = Left$(fname, Len(fname)-4)
End Function

Sub Parse (strin$, strout$, xout As Variant, delim$)
    Dim i
    strin$ = LTrim$((strin$))
    i = InStr(strin$, delim$)
    If i > 0 Then
        strout$ = LTrim$(Mid$(strin$, 1, i - 1))
        strin$ = Mid$(strin$, i + Len(delim$))
    Else
        strout$ = strin$
        strin$ = ""
    End If
    xout = Val(strout$)
End Sub

Function FileExists(ByVal f$) As Boolean
    On Error GoTo NoFile
    FileExists = FileLen(f$) > 0
    Exit Function
NoFile:
    FileExists = False
End Function

Function cRnd(ByVal v, ByVal n) As String
    cRnd = Str$(Round(v, n))
End Function
